"use strict";

// ------------ Basic Dependencies -------------------------------------------
var express = require('express'),
	session = require('express-session'),
	path = require('path'),
	favicon = require('serve-favicon'),
	logger = require('morgan'),
	cookieParser = require('cookie-parser'),
	bodyParser = require('body-parser'),
	requireDir = require('require-dir'),
	connection = require('express-myconnection'),
	bcrypt = require('bcrypt-nodejs'),
	mysql = require('mysql');
// ----------------------------------------------------------------------------
var app = express();

// General config
app.set('trust proxy', true); // Trust X-Forwarded-* headers

// Database connection
app.use(
	connection('mysql', {
		host: 'localhost',
		user: 'node',
		password: 'node',
		port: 3306,
		database: 'node'
	}, 'request')
);

// view engine setup
var consolidate = require('consolidate');
var dust = require('dustjs-linkedin');
app.engine('dust', consolidate.dust);
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'dust');

// General app setup
app.use(favicon(__dirname + '/public/favicon.ico'));
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(express.static(path.join(__dirname, 'public'), {redirect:false}));

// Route mapping
// Routes are prefixed by the filename,
// unless the filename is index.
//
// A '/' route in the users file becomes the '/users/' route.
var routes = requireDir('routes');
Object.keys(routes).forEach(function(route) {
	var path = (route != 'index')
		? '/' + route
		: '/';
	app.use(path, routes[route]);
});

// catch 404 and forward to error handler
app.use(function(req, res, next) {
	var err = new Error('Not Found');
	err.status = 404;
	next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
	app.use(function(err, req, res, next) {
		res.status(err.status || 500);
		res.render('error', {
			message: err.message,
			error: err
		});
	});
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
	res.status(err.status || 500);
	res.render('error', {
		message: err.message,
		error: {}
	});
});


module.exports = app;